@@ -66,7 +66,7 @@ void          gtk_stock_item_free (GtkStockItem       *item);
 void          gtk_stock_set_translate_func (const gchar      *domain,
 					    GtkTranslateFunc  func,
 					    gpointer          data,
-					    GtkDestroyNotify  notify);
+					    GDestroyNotify    notify);
 
 /* Stock IDs (not all are stock items; some are images only) */
 #define GTK_STOCK_DIALOG_AUTHENTICATION \
@@ -148,6 +148,7 @@ void          gtk_stock_set_translate_func (const gchar      *domain,
 #define GTK_STOCK_PREFERENCES      "gtk-preferences"
 #define GTK_STOCK_PRINT            "gtk-print"
 #define GTK_STOCK_PRINT_ERROR      "gtk-print-error"
+#define GTK_STOCK_PRINT_PAUSED     "gtk-print-paused"
 #define GTK_STOCK_PRINT_PREVIEW    "gtk-print-preview"
 #define GTK_STOCK_PRINT_REPORT     "gtk-print-report"
 #define GTK_STOCK_PRINT_WARNING    "gtk-print-warning"
