@@ -24,6 +24,10 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_TEXT_BUFFER_H__
 #define __GTK_TEXT_BUFFER_H__
 
@@ -49,9 +53,9 @@ G_BEGIN_DECLS
  */
 typedef enum
 {
-  GTK_TEXT_BUFFER_TARGET_INFO_BUFFER_CONTENTS = G_MAXUINT - 0,
-  GTK_TEXT_BUFFER_TARGET_INFO_RICH_TEXT       = G_MAXUINT - 1,
-  GTK_TEXT_BUFFER_TARGET_INFO_TEXT            = G_MAXUINT - 2
+  GTK_TEXT_BUFFER_TARGET_INFO_BUFFER_CONTENTS = - 1,
+  GTK_TEXT_BUFFER_TARGET_INFO_RICH_TEXT       = - 2,
+  GTK_TEXT_BUFFER_TARGET_INFO_TEXT            = - 3
 } GtkTextBufferTargetInfo;
 
 typedef struct _GtkTextBTree GtkTextBTree;
@@ -71,20 +75,20 @@ struct _GtkTextBuffer
 {
   GObject parent_instance;
 
-  GtkTextTagTable *tag_table;
-  GtkTextBTree *btree;
+  GtkTextTagTable *GSEAL (tag_table);
+  GtkTextBTree *GSEAL (btree);
 
-  GSList *clipboard_contents_buffers;
-  GSList *selection_clipboards;
+  GSList *GSEAL (clipboard_contents_buffers);
+  GSList *GSEAL (selection_clipboards);
 
-  GtkTextLogAttrCache *log_attr_cache;
+  GtkTextLogAttrCache *GSEAL (log_attr_cache);
+
+  guint GSEAL (user_action_count);
 
-  guint user_action_count;
-  
   /* Whether the buffer has been modified since last save */
-  guint modified : 1;
+  guint GSEAL (modified) : 1;
 
-  guint has_selection : 1;
+  guint GSEAL (has_selection) : 1;
 };
 
 struct _GtkTextBufferClass
@@ -103,7 +107,7 @@ struct _GtkTextBufferClass
   void (* insert_child_anchor)   (GtkTextBuffer      *buffer,
                                   GtkTextIter        *pos,
                                   GtkTextChildAnchor *anchor);
-  
+
   void (* delete_range)     (GtkTextBuffer *buffer,
                              GtkTextIter   *start,
                              GtkTextIter   *end);
@@ -247,6 +251,9 @@ GtkTextChildAnchor *gtk_text_buffer_create_child_anchor (GtkTextBuffer *buffer,
                                                          GtkTextIter   *iter);
 
 /* Mark manipulation */
+void           gtk_text_buffer_add_mark    (GtkTextBuffer     *buffer,
+                                            GtkTextMark       *mark,
+                                            const GtkTextIter *where);
 GtkTextMark   *gtk_text_buffer_create_mark (GtkTextBuffer     *buffer,
                                             const gchar       *mark_name,
                                             const GtkTextIter *where,
@@ -374,7 +381,7 @@ gboolean        gtk_text_buffer_get_selection_bounds    (GtkTextBuffer *buffer,
                                                          GtkTextIter   *end);
 gboolean        gtk_text_buffer_delete_selection        (GtkTextBuffer *buffer,
                                                          gboolean       interactive,
-                                                         gboolean       default_editable);                                                    
+                                                         gboolean       default_editable);
 
 /* Called to specify atomic user actions, used to implement undo */
 void            gtk_text_buffer_begin_user_action       (GtkTextBuffer *buffer);
